;/********************************/;
;/* Demo Scankey 4x3 For ET-EXP4 */;
;/* Files Name  : LAB4-EX2.ASM   */;
;/* CPU Control : 68HC811E2      */;
;/* Assembler   : AS11NEW.EXE    */;
;/*  (Scankey & Display on LCD)  */;
;/********************************/;
;
;/* Port Scankey Matrix 4x3 (Port-C)
PORT_KEY    EQU     $1003               ; Port-C
CTRL_KEY    EQU     $1007               ; DDRC 0=In,1=Out
;
;/* Port For Control LCD (Port-B)
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)

            ORG     $00
SCAN_BUF    RMB     1                   ; Colum Scan Control
KEY_FLAG    RMB     1                   ; 1 = old ,0 = new

            ORG     $F800               ; Start Monitor of 68H1811E2
START       LDS     #$FF                ; Stack on Last Internal RAM
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
            CLR     KEY_FLAG
            LDAA    #%00001111          ; PCH=In,PCL=Out
            STAA    CTRL_KEY
            ;
WAIT_KEY    JSR     SCAN_KEY            ; Scankey 4X3
            CMPA    #$FF
            BEQ     WAIT_KEY            ; Loop Until Keypress
            ;
DSP_KEY1    CMPA    #$09                ; Convert to ASCII
            BHI     DSP_KEY2
            ORAA    #$30                ; Convert 0-9 to ASCII
            JSR     WR_LCD
            BRA     WAIT_KEY
            ;
DSP_KEY2    SBCA    #$09                ; Convert A-F to ASCII
            ORAA    #$40
            JSR     WR_LCD
            BRA     WAIT_KEY

;/******************************/;
;/* Scan Keyboard Matrix (4x3) */;
;/* PC0..PC3 : Colum (Out)     */;
;/* PC4..PC7 : Row (In)        */;
;/* Output   : ACC (Code 0..B) */;
;/*          : FFH (Not Press) */;
;/******************************/;
;
SCAN_KEY    LDAB    #3                  ; Counter for Scan 4x3
            LDAA    #%00000001          ; Data For Colum Scan
            STAA    SCAN_BUF
            ;
SCAN1       LDAA    SCAN_BUF
            COMA
            STAA    PORT_KEY            ; Scan Colum (PC0..PC3)
            LDAA    PORT_KEY            ; In Row (PC4..PC7)
            ANDA    #$F0
            CMPA    #$F0                ; Check row only (PC0..PC3)
            BNE     KEY_PRES            ; Jump if keypress
            LDAA    SCAN_BUF
            ASLA
            STAA    SCAN_BUF            ; Next Colum Scan
            DECB
            BNE     SCAN1
            LDAA    #$FF                ; Not Keypress
            CLR     KEY_FLAG            ; Reset Press
            BRA     END_SCAN
            ;
KEY_PRES    JSR     DLY_10MS            ; Debounce key
            LDAA    PORT_KEY            ; In Row (PC4..PC7)
            ANDA    #$F0
            CMPA    #$F0                ; Check row only (PC4..PC7)
            BNE     KEY_CODE            ; YES keypress
            BRA     SCAN_KEY            ; NO keybounce or noise
            ;
KEY_CODE    JSR     KEY_NEW
            BCC     SCAN1               ; Ignore if old key
            LDAB    SCAN_BUF
            ABA                         ; Mix Row & Colum
            ;
            LDY     #KEY_TAB            ; Row & Colum Code
            LDAB    #12                 ; 12 Key Count
CODE1       CMPA    0,Y
            BEQ     CODE2
            INY
            DECB
            BNE     CODE1
            LDAA    #$FF                ; Not Math in 12 Key
            BRA     END_SCAN
CODE2       DECB
            TBA                         ; Keycode (0..B)
END_SCAN    RTS

;/******************************/;
;/* Check Old & New key Status */;
;/* Out Status = Carry flag    */;
;/* Carry Flag(0) is old press */;
;/* Carry Flag(1) is new press */;
;/******************************/;
;
KEY_NEW     PSHA
            CLC                         ; old key Status
            LDAA    KEY_FLAG
            BNE     END_NEW             ; Not Zero
            INCA                        ; old key in Next Time
            STAA    KEY_FLAG
            SEC                         ; New Key Status
END_NEW     PULA
            RTS

;/**************************/;
;/* Table Code For Key 4x3 */;
;/* 1(E1H)  2(E2H)  3(E4H) */;
;/* 4(D1H)  5(D2H)  6(D4H) */;
;/* 7(B1H)  8(B2H)  9(D4H) */;
;/* *(71H)  0(72H)  #(74H) */;
;/**************************/;
;
KEY_TAB     FCB     $74,$71,$B4,$B2     ; #    *    9    8
            FCB     $B1,$D4,$D2,$D1     ; 7    6    5    4
            FCB     $E4,$E2,$E1,$72     ; 3    2    1    0

;/*******************/;
;/* Delay Time 10mS */;
;/* E-Clock = 2 MHz */;
;/*******************/;
;
DLY_10MS    PSHX
            LDX     #$0D06              ; 10 mSec counter
DLY10       DEX
            BNE     DLY10
            PULX
            RTS

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PB0   */;
;/* -> E    = PB2   */;
;/* -> D4-7 = PB4-7 */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0F                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor on,Blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
            LDAA    #$01                ; Clear Display
            JSR     WR_INS              ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      PSHB
            LDAB    #20                 ; 100uS counter
BUSY        DECB
            BNE     BUSY
            PULB
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0632              ; 5 mSec counter
INS_DL1     DEX
            BNE     INS_DL1
            PULX
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0632              ; 5 mSec counter
INS_DL2     DEX
            BNE     INS_DL2
            PULX
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

            ORG     $FFFE               ; Reset Vector
            FDB     START

            END


