;/*******************************/;
;/* Demo Speaker For ET-EXP4    */;
;/* Files Name  : LAB5-EX5.ASM  */;
;/* CPU Control : 68HC811E2     */;
;/* Assembler   : AS11NEW.EXE   */;
;/* -Timer Run Interrupt Mode   */;
;/* -Used OC3 : Gen Music Note  */;
;/* (Generate Sound on Port-A)  */;
;/*******************************/;
;
IOREGS      EQU     $1000               ; Start of I/O Register
OC1M        EQU     IOREGS+$0C          ; OC1 Mask Bits
OC1D        EQU     IOREGS+$0D          ; OC1 Data Bits??
TCNT        EQU     IOREGS+$0E          ; Timer Count Register
TOC3        EQU     IOREGS+$1A          ; Output Compare #3
TCTL1       EQU     IOREGS+$20          ; Timer Control Register #1
TMSK1       EQU     IOREGS+$22          ; Timer Mask #1
TFLG1       EQU     IOREGS+$23          ; Timer Flags #1
TMSK2       EQU     IOREGS+$24          ; Timer Mask #2
TFLG2       EQU     IOREGS+$25          ; Timer Flags #2

            ORG     $00
SONG_PTR    RMB     2                   ; Music Note Pointer
SONG_CNT    RMB     2                   ; Song Delay Count
NOTE_CNT    RMB     1                   ; Note/Output Count
;
FREQ_CNT    RMB     2                   ; Timer 1/2 Cycle Count
FREQ_OC3    RMB     2

            ORG     $F800               ; Start Monitor of 68HC811E2
MAIN        LDS     #$FF                ; Stack = Last Internal RAM
            LDD     #0                  ; SONG_CNT = 0
            STD     SONG_CNT
            LDAB    TMSK2
            ORAB    #%01000000          
            STAB    TMSK2               ; Enable RTI Interrupt
            LDAB    #%01000000
            STAB    TFLG2               ; Clear RTIF Flag
            ;
            LDAB    TMSK1
            ORAB    #%00100000
            STAB    TMSK1               ; Enable OC3I
            LDAB    #%00100000
            STAB    TFLG1               ; Clear OC3F
            CLI                         ; Enable Global Interrupt
            ;
NEW_SONG    LDAB    #%00100000
            STAB    TFLG1               ; Clear OC3F
            LDD     #0                  ; Start 1st Music Note
            STD     SONG_PTR
            ;
            LDAB    #32
            STAB    NOTE_CNT            ; 8-Note on OC3(PA5)
            ;
SONG_OC3    JSR     GET_NOTE            ; Get Note & Delay
            LDD     FREQ_CNT
            STD     FREQ_OC3
            LDD     FREQ_OC3            ; FREQ_OC3 = 0 ?
            CPD     #0
            BNE     ENA_OC3
            JMP     DIS_OC3
            ;
ENA_OC3     LDD     TCNT
            ADDD    FREQ_OC3
            STD     TOC3
            LDAB    TCTL1
            ORAB    #%00010000
            STAB    TCTL1               ; Toggle OC3 Pin
            JMP     CHK_OC3
            ;
DIS_OC3     CLRB                        ; No Active Output
            STAB    TCTL1               ; Disconnect OC2-5
            STAB    OC1M                ; Disconnect PA7
            STAB    OC1D                ; PA7 = 0
            ;
CHK_OC3     LDD     SONG_CNT            ; Loop Until SONG_CNT = 0
            CPD     #0
            BNE     CHK_OC3
            DEC     NOTE_CNT
            BNE     SONG_OC3
            JMP     NEW_SONG

;/*********************************/;
;/* Get Frequency & Delay of Note */;
;/* Input  : SONG_PTR(Note Table) */;
;/* Output : FREQ_CNT(Note Freq.) */;
;/*        : SONG_CNT(Note Delay) */;
;/*        : SONG_PTR=SONG_PTR+2  */;
;/*********************************/;
;
GET_NOTE    LDD     SONG_PTR            ; Get Note Pointer
            ADDD    #TAB_SONG
            PSHB
            PSHA
            PULX
            LDAA    #2
            LDAB    0,X                 ; B = Note
            MUL
            ADDD    #TAB_NOTE
            PSHB
            PSHA
            PULX                        ; X = Note Pointer
            LDD     0,X
            STD     FREQ_CNT            ; Save Note Frequency
            LDD     #1
            ADDD    SONG_PTR
            STD     SONG_PTR            ; SONG_PTR = SONG_PTR+1
            ;
            ;/* Get Delay of Note (Long-Time) */;
            LDD     SONG_PTR            ; get duration
            ADDD    #TAB_SONG
            PSHB
            PSHA
            PULX
            LDAA    #2
            LDAB    0,X                 ; B = Delay
            MUL
            ADDD    #TAB_DELAY
            PSHB
            PSHA
            PULX                        ; X = Delay Pointer
            LDD     0,X
            STD     SONG_CNT            ; Save Note Delay
            LDD     #1
            ADDD    SONG_PTR
            STD     SONG_PTR            ; SONG_PTR = SONG_PTR+1
            RTS

;/*****************/;
;/* RTI Interrupt */;
;/*****************/;
;
RTI_INT     LDD     SONG_CNT            ; Get Note
            CPD     #0
            BHI     DEC_NOTE            ; If >= 0 Jump (*+5)
            JMP     CLR_FLAG
DEC_NOTE    LDD     SONG_CNT            ; SONG_CNT = SONG_CNT-1
            SUBD    #1
            STD     SONG_CNT
CLR_FLAG    LDD     #%01000000          ; Clear RTIF flag
            STAB    TFLG2
            RTI

;/******************/;
;/* TOC3 Interrupt */;
;/******************/;
;
OC3_INT     LDD     TOC3                ; Read TOC3
            ADDD    FREQ_OC3            ; TOC3 = TOC3 + FREQ_OC3
            STD     TOC3
            LDD     #%00100000          ; Clear OC3F Flag
            STAB    TFLG1
            RTI

;/********************/;
;/* Table of Pointer */;
;/* For Note & Delay */;
;/* Song of All Note */;
;/********************/;
;
TAB_SONG    FCB     00,1,01,5,02,5,03,5
            FCB     04,5,05,5,06,5,07,5
            FCB     00,1,08,5,09,5,10,5
            FCB     11,5,12,5,13,5,14,5
            FCB     00,1,15,5,16,5,17,5
            FCB     18,5,19,5,20,5,21,5
            FCB     00,1,22,5,23,5,24,5
            FCB     25,5,26,5,27,5,28,5
            FCB     00,1,29,5,30,5,31,5
            FCB     32,5,33,5,34,5,35,5
            FCB     255                 ; End of Music

;/**************************/;
;/* Table of Standard Note */;
;/**************************/;
;
TAB_NOTE    FDB     0000                ; No Sound
            ;
            FDB     3822                ; Note C  (261.624 Hz) Octave4
            FDB     3405                ; Note D  (293.664 Hz)
            FDB     3034                ; Note E  (329.624 Hz)
            FDB     2863                ; Note F  (349.232 Hz)
            FDB     2551                ; Note G  (391.992 Hz)
            FDB     2273                ; Note A  (440.000 Hz)
            FDB     2025                ; Note B  (493.880 Hz)
            ;
            FDB     1911                ; Note C  (523.248 Hz) Octave5
            FDB     1703                ; Note D  (587.238 Hz)
            FDB     1517                ; Note E  (659.248 Hz)
            FDB     1432                ; Note F  (698.464 Hz)
            FDB     1276                ; Note G  (783.984 Hz)
            FDB     1136                ; Note A  (880.000 Hz)
            FDB     1012                ; Note B  (987.760 Hz)
            ;
            FDB     956                 ; Note C  (1046.496 Hz) Octave6
            FDB     851                 ; Note D  (1174.656 Hz)
            FDB     758                 ; Note E  (1318.496 Hz)
            FDB     716                 ; Note F  (1396.928 Hz)
            FDB     638                 ; Note G  (1567.968 Hz)
            FDB     568                 ; Note A  (1760.000 Hz)
            FDB     506                 ; Note B  (1975.520 Hz)
            ;
            FDB     478                 ; Note C  (2092.992 Hz) Octave7
            FDB     426                 ; Note D  (2349.312 Hz)
            FDB     379                 ; Note E  (2636.992 Hz)
            FDB     358                 ; Note F  (2793.856 Hz)
            FDB     319                 ; Note G  (3135.936 Hz)
            FDB     284                 ; Note A  (3520.000 Hz)
            FDB     253                 ; Note B  (3951.040 Hz)
            ;
            FDB     239                 ; Note C  (4185.984 Hz) Octave8
            FDB     213                 ; Note D  (4698.624 Hz)
            FDB     190                 ; Note E  (5273.984 Hz)
            FDB     179                 ; Note F  (5587.712 Hz)
            FDB     161                 ; Note G  (6217.872 Hz)
            FDB     142                 ; Note A  (7040.000 Hz)
            FDB     127                 ; Note B  (7902.080 Hz)

;/********************/;
;/* Note Delay Table */;
;/********************/;
;
TAB_DELAY   FDB     0,255,190,127,94
            FDB     63,47,31,23,16

            ORG     $FFE4
            FDB     OC3_INT             ; OC3 Vector
            ORG     $FFF0
            FDB     RTI_INT             ; RTI Vector
            ORG     $FFFE               ; Reset Vector
            FDB     MAIN

            END

