;/*******************************/;
;/* Demo RS232/422 For ET-EXP4  */;
;/* Files Name  : LAB6-EX1.ASM  */;
;/* CPU Control : 68HC811E2     */;
;/* Assembler   : AS11NEW.EXE   */;
;/*  (Send & Receive RS232/422) */;
;/*******************************/;
;
BAUD        EQU     $102B               ; SCI Baud rate control Reg.
SCCR1       EQU     $102C               ; SCI Control Register1
SCCR2       EQU     $102D               ; SCI Control Register2
SCSR        EQU     $102E               ; SCI Status Register
SCDR        EQU     $102F               ; SCI Data (RDR=Read,TDR=Write)
;
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)

            ORG     $F800               ; Start at Monitor 68HC811E2
START       LDS     #$FF                ; Stack = Last Internal RAM
            JSR     INIT_SCI            ; Initial SCI 9600,N,8,1
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            LDAA    #'>'
            JSR     TX_SCI              ; Start Prompt
            ;
LOOP        JSR     RX_SCI              ; Wait to Receive
            JSR     TX_SCI              ; Echo Receive
            CMPA    #$0D                ; Receive = Enter ?
            BEQ     CLR_DSP
            JSR     WR_LCD              ; Display Receive
            BRA     LOOP
CLR_DSP     JSR     CLR_LCD             ; If Enter Clear Display
            BRA     LOOP

;/***************************************/;
;/* Initial the SCI Port for 68HC11 CPU */;
;/* Baud rate  : Crystal Oscilator 8MHz */;
;/* Format SCI : Data 8 bit, 1 Stop bit */;
;/***************************************/;
;
INIT_SCI    LDAA    #%00110000          ; Set Baud rate 9600
            ;LDAA    #%00110001          ; Set Baud rate 4800
            ;LDAA    #%00110010          ; Set Baud rate 2400
            ;LDAA    #%00110011          ; Set Baud rate 1200
            STAA    BAUD
            LDAA    #$00
            STAA    SCCR1               ; Set 8bit data,1 Stop bit
            LDAA    #$0C                ; Enable RXD,TXD pin
            STAA    SCCR2
            RTS

;/********************************/;
;/* Transmit ASCII 1byte to SCI  */;
;/* Input    : ACCA (ASCII Code) */;
;/* Output   : ASCII to SCI Port */;
;/* Register : ACCA,ACCB         */;
;/********************************/;
;
TX_SCI      PSHB
TX_SCI1     LDAB    SCSR                ; Read Status
            BITB    #$80
            BEQ     TX_SCI1             ; Wait Ready to Send (TDRE=1)
            STAA    SCDR                ; Send character to SCI
            PULB
            RTS

;/*************************/;
;/* Receive data from SCI */;
;/* INPUT  : SCI Port     */;
;/* OUTPUT : ACCA         */;
;/*************************/;
;
RX_SCI      LDAA    SCSR                ; Read status reg
            ANDA    #$20                ; Check RDRF
            BEQ     RX_SCI              ; Loop if no Data
            LDAA    SCDR                ; Read Data
            RTS

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PB0   */;
;/* -> E    = PB2   */;
;/* -> D4-7 = PB4-7 */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0F                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor on,blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
CLR_LCD     LDAA    #$01                ; Clear Display
            JSR     WR_INS              ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      PSHB
            LDAB    #20                 ; 100uS counter
BUSY        DECB
            BNE     BUSY
            PULB
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0632              ; 5 mSec counter
INS_DLY     DEX
            BNE     INS_DLY
            PULX
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

            ORG     $FFFE
            FDB     START

            END

