;/*******************************/;
;/* Demo DC-Motor For ET-EXP4   */;
;/* Files Name  : LAB9-EX2.ASM  */;
;/* CPU Control : 68HC811E2     */;
;/* Assembler   : AS11NEW.EXE   */;
;/*  (Auto DAC Control Motor)   */;
;/*******************************/;
;
;/* Port LED Status (Port-C)
PORT_LED    EQU     $1003                   ; Port-C
CTRL_LED    EQU     $1007                   ; DDRC 0=In,1=Out
;
;/* Port For Control DAC
PORT_DAC    EQU     $1004                   ; DAC Output(Port-B)
;
;/* Port Control Motor
PORT_MTR    EQU     $1000                   ; DC Motor Control(Port-A)
PACTL       EQU     $1026                   ; Direction Port-A
;
TIC1        EQU     $1010                   ; Input Capture #1
TCTL1       EQU     $1020                   ; Timer Control Register #1
TCTL2       EQU     $1021                   ; Timer Control Register #2
TMSK1       EQU     $1022                   ; Timer Mask #1
TFLG1       EQU     $1023                   ; Timer Flags #1
TMSK2       EQU     $1024                   ; Timer Mask #2
TFLG2       EQU     $1025                   ; Timer Flags #2
;
SPEED_MAX   EQU     20                      ; 20 Round : 1 Second

            ORG     $00
SPEED       RMB     1                       ; Speed Motor
SPEED_CNT   RMB     1                       ; Speed Count : Time
SEC_CNT     RMB     1                       ; 1 Second Time Count
AUTO_STOP   RMB     1

            ORG     $F800                   ; Start Monitor of 68H1811E2
START       LDS     #$FF                    ; Stack on Last Internal RAM
            LDAA    #%10001000
            STAA    PACTL                   ; PA3,PA7 = Output Port
            LDAA    #%00010000          
            STAA    PORT_MTR                ; PA4 = "1" and PA5 = "0"
            LDAA    #%11111111              ; PCH,PCL=Out
            STAA    CTRL_LED
            ;
            LDAA    TMSK2
            ORAA    #%01000000
            STAA    TMSK2                   ; Enable RTI Interrupt
            LDAA    #%01000000
            STAA    TFLG2                   ; Clear RTIF Flag
            ;
            LDAA    #%00000010              ; Capture IC3 on Falling Edge
            STAA    TCTL2
            LDAA    #%00000001
            STAA    TFLG1                   ; Clear IC3F
            CLI                             ; Enable Global Interrupt
            ;
            CLRA
            STAA    PORT_LED
            LDAA    #$FF                    ; Start Motor
            STAA    PORT_DAC
            ;
            LDX     #$1000                  ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            ;
            LDAA    #$7F                    ; Speed 50%
            STAA    SPEED
            STAA    PORT_DAC
            ;
            ;/* Check Speed : 1 Second
CTL_MTR0    CLR     SPEED_CNT
            LDAA    #244                    ; 1000mS / 4.10mS = 1 Second
            STAA    SEC_CNT
            CLR     AUTO_STOP
            ;
CTL_MTR1    LDAA    AUTO_STOP
            BNE     CTL_MTR2
            LDAA    TFLG1
            ANDA    #%00000001
            BEQ     CTL_MTR1                ; Wait Sensor
            INC     SPEED_CNT
            LDAA    #%00000001
            STAA    TFLG1                   ; Clear IC1F
            BRA     CTL_MTR1                ; Wait Time-Out
            ;
CTL_MTR2    LDAA    SPEED_CNT
            CMPA    #SPEED_MAX              ; Check Speed
            BNE     CTL_MTR3
            LDAA    #%00011000              ; Speed OK Status
            STAA    PORT_LED
            BRA     CTL_MTR0                ; Speed OK
            ;
CTL_MTR3    BLO     CTL_MTR5                ; Lessthan Speed MAX
            LDAA    #%11000000              ; Speed Over Status
            STAA    PORT_LED
            LDAA    SPEED
            BNE     CTL_MTR4
            BRA     CTL_MTR0                ; MIN Adjust
            ;
CTL_MTR4    DEC     SPEED                   ; Speed Over Limit
            LDAA    SPEED
            STAA    PORT_DAC
            BRA     CTL_MTR0
            ;
CTL_MTR5    LDAA    #%00000011              ; Speed Under Status
            STAA    PORT_LED
            LDAA    SPEED
            CMPA    #$FF
            BNE     CTL_MTR6
            BRA     CTL_MTR0                ; MAX Adjust
            ;
CTL_MTR6    INC     SPEED                   ; Speed Under Limit
            LDAA    SPEED
            STAA    PORT_DAC
            BRA     CTL_MTR0

;/*****************/;
;/* RTI Interrupt */;
;/*   4.10 mSEC   */;
;/*****************/;
;
RTI_INT     LDAA    AUTO_STOP
            BNE     CLR_FLAG                ; Time-Out Ready
            DEC     SEC_CNT                 ; Check 1Second ?
            BNE     CLR_FLAG
            INC     AUTO_STOP
CLR_FLAG    LDD     #%01000000              ; Clear RTIF flag
            STAB    TFLG2
            RTI

            ORG     $FFF0
            FDB     RTI_INT                 ; RTI Vector
            ORG     $FFFE               
            FDB     START                   ; Reset Vector

            END



